﻿Imports System.IO

Public Class MainForm

    Private Problems As New List(Of Problem)
    Private ProblemIndex As Integer = -1
    Private ConfigFile As String = Application.StartupPath + "\probs.txt"
    Private LastModify As Date
    Private ReadError As Integer

#Region " Events "

#Region " MainForm "

    ''' <summary>
    ''' Load problems and display balloon if there are any
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.LoadProblems()
        If Me.Problems.Count > 0 Then
            Me.DisplayProblemBaloon()
        End If
    End Sub

    ''' <summary>
    ''' Toggle taskbar depending on form state
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainForm_Resize(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        'Required to stop stack overflow at startup
        If Not Me.Created Then Return
        Me.ShowInTaskbar = Me.WindowState <> FormWindowState.Minimized
    End Sub

#End Region

#Region " Buttons "

    ''' <summary>
    ''' Clears Form and adjusts gui for new input
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub B_New_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles B_New.Click
        Me.ClearForm()
        Me.ProblemIndex = -1
        Me.GroupBox.Enabled = True
        Me.TB_CreatedAt.Text = Now.ToString()
        Me.TB_Summary.Focus()
        Me.B_Delete.Enabled = False
        Me.B_Save.Enabled = True
    End Sub

    ''' <summary>
    ''' Increments problem index
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub B_Next_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles B_Next.Click
        If Me.ProblemIndex + 1 >= Me.Problems.Count Then
            Me.ProblemIndex = 0
        Else
            Me.ProblemIndex += 1
        End If
        Me.UpdateFormWithProblem()
    End Sub

    ''' <summary>
    ''' Sets Ack time to now and adjusts gui
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub B_Ack_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles B_Ack.Click
        Me.Problems(Me.ProblemIndex).Acknowledge = Now
        B_Ack.Enabled = False
    End Sub

    ''' <summary>
    ''' Saves changes to current or adds a new then saves, auto aks
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub B_Save_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles B_Save.Click
        If TB_CreatedAt.Text = "" Or TB_CreatedBy.Text = "" Or TB_Duration.Text = "" Or TB_Message.Text = "" Or TB_Summary.Text = "" Then
            MsgBox("Please fill all fields", MsgBoxStyle.Information, "More Information")
            Return
        End If

        If Me.ProblemIndex = -1 Then
            Dim NewProb As New Problem
            NewProb.ID = Now.ToString
            NewProb.FillValues()
            Me.Problems.Add(NewProb)
            Me.ProblemIndex = Me.Problems.Count - 1
        Else
            Me.Problems(Me.ProblemIndex).FillValues()
        End If
        Me.Problems(Me.ProblemIndex).Acknowledge = Now
        Me.SaveFile()
        Me.UpdateFormWithProblem()
    End Sub

    ''' <summary>
    ''' Displays confirmation and then deletes
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub B_Delete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles B_Delete.Click
        'TODO:Add a delete on ack before removal
        If MsgBox("Please update the issue with a resolution before deletion." + vbCrLf + "Confirm Delete", MsgBoxStyle.Question Or MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
            Me.Problems.RemoveAt(Me.ProblemIndex)
            Me.SaveFile()
            Me.ClearFormOrSetIndexToFirst()
            Me.UpdateFormWithProblem()
        End If
    End Sub

    ''' <summary>
    ''' Displays About box
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub B_About_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles B_About.Click
        AboutBox.ShowDialog(Me)
    End Sub

    ''' <summary>
    ''' Dispose the Task bar icon
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainForm_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        Me.NI_Notify.Dispose()
        Me.NI_Notify = Nothing
    End Sub

#End Region

#Region " Timers "

    ''' <summary>
    ''' Checks File mod date on message file and loads if newer than last load
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TimerCheckFile_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TimerCheckFile.Tick
        If Me.LastModify <> File.GetLastWriteTime(ConfigFile) Then
            Me.LoadProblems()
        End If
    End Sub

    ''' <summary>
    ''' Displays balloon
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TimerNotify_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TimerNotify.Tick
        Me.DisplayProblemBaloon()
    End Sub

#End Region

    ''' <summary>
    ''' Clicking on notify icon displays the form
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub NI_Notify_MouseClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles NI_Notify.MouseClick
        If Me.WindowState = FormWindowState.Minimized Then
            Me.WindowState = FormWindowState.Normal
        End If
    End Sub

#End Region

#Region " IO "

    ''' <summary>
    ''' Saves the problems to file
    ''' </summary>
    ''' <remarks>Need to put in a retry on fail if two people save at the same time</remarks>
    Private Sub SaveFile()
        Dim sw As StreamWriter = Nothing
        Try
            sw = New StreamWriter(Me.ConfigFile, False)
            Me.Problems.ForEach(Sub(x) x.Save(sw))
            sw.Close()
        Catch ex As Exception
            MsgBox("Error writting file, Please try again", MsgBoxStyle.Information, "Error")
        Finally
            If Not sw Is Nothing Then sw.Dispose()
            sw = Nothing
        End Try
    End Sub

    ''' <summary>
    ''' Loads Problems from text file.
    ''' Removes remotley deleted problems from memory
    ''' </summary>
    ''' <remarks>Creates a file if not present, counts failures on error to a max then fails program if exceeded</remarks>
    Private Sub LoadProblems()

        'Creates a new, empty file (first run)
        If Not File.Exists(ConfigFile) Then
            Dim sw As StreamWriter = Nothing
            Try
                sw = File.CreateText(ConfigFile)
                sw.Close()
                Me.LastModify = File.GetLastWriteTime(ConfigFile)
            Catch ex As Exception
            Finally
                If Not sw Is Nothing Then sw.Dispose()
                sw = Nothing
            End Try
            Return
        End If

        'Reads file looking for headers "[" and passes to problem object for reading of values
        'Updates all as removed and cancels if found
        'Unsets aks on updates found
        'Sets a display flag if new issue found
        Dim display As Boolean
        Dim sr As StreamReader = Nothing
        Try

            sr = New StreamReader(ConfigFile)
            Dim line As String
            Me.Problems.ForEach(Sub(x) x.Removed = True)

            Do While Not sr.EndOfStream

                line = sr.ReadLine.Trim()

                'Skip blanks
                If line = "" Then Continue Do

                'Headers
                If line.StartsWith("[") Then

                    Dim ID As String
                    line = line.Replace("[", "")
                    line = line.Replace("]", "")
                    ID = line.Trim()

                    'Find existing problem or create new
                    Dim existing As Problem = Me.Problems.Find(Function(x) x.ID = ID)
                    If existing Is Nothing Then
                        Dim newProb As New Problem
                        newProb.Load(sr, ID)
                        Me.Problems.Add(newProb)
                        display = True
                    Else
                        'set to not remove
                        existing.Removed = False
                        existing.Load(sr, ID)
                        'Reset ack if this is an update
                        If existing.Acknowledge < existing.Updated Then
                            existing.Summary = "* " + existing.Summary
                            existing.Acknowledge = Date.MinValue
                            display = True
                        End If
                    End If
                End If
            Loop
            sr.Close()
            Me.LastModify = File.GetLastWriteTime(ConfigFile)
            Me.ReadError = 0
        Catch ex As Exception
            Me.ReadError += 1
        Finally
            If Not sr Is Nothing Then sr.Dispose()
            sr = Nothing
        End Try

        'display error is erroring and exit
        If Me.ReadError >= 10 Then
            MsgBox("Error reading message file, delete or fix ""Probs.txt"" in """ + vbCrLf + Application.StartupPath + vbCrLf + """ to reset", MsgBoxStyle.Critical, "Error")
            End
        End If

        'If viewing an issue
        If Me.WindowState <> FormWindowState.Minimized And Me.ProblemIndex > -1 Then
            'If issue being viewed has been removed
            If Me.Problems(Me.ProblemIndex).Removed Then
                MsgBox("This issue has been removed", vbInformation)
                Me.ProblemIndex = -1
                Me.ClearFormOrSetIndexToFirst()
                Me.UpdateFormWithProblem()
            End If
        End If

        'Remove all that are no longer present in config file
        Me.Problems.RemoveAll(Function(x) x.Removed = True)

        'If there is some update or a new the display the baloon
        If display Then DisplayProblemBaloon()

        'If form is mini
        If Me.WindowState = FormWindowState.Minimized Then
            'set problems to first unack
            Dim firstUnAked As Problem = Me.Problems.Find(Function(x) x.Acknowledge = Date.MinValue)
            If Not firstUnAked Is Nothing Then Me.ProblemIndex = Me.Problems.IndexOf(firstUnAked)
            Me.ClearFormOrSetIndexToFirst()
            Me.UpdateFormWithProblem()
        End If

    End Sub

#End Region

#Region " GUI "

    ''' <summary>
    ''' Changes current index based on
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ClearFormOrSetIndexToFirst()

        If Me.ProblemIndex >= Me.Problems.Count Then
            Me.ProblemIndex = -1
        End If

        Me.GroupBox.Enabled = Me.Problems.Count > 0
        If Me.ProblemIndex = -1 Then
            If Me.Problems.Count = 0 Then
                Me.ClearForm()
            Else
                Me.ProblemIndex = 0
            End If
        End If

    End Sub

    ''' <summary>
    ''' Writes the current problem to the text boxes, or adjusts gui for a new form
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub UpdateFormWithProblem()
        If Me.ProblemIndex = -1 Then
            B_Ack.Enabled = False
            B_Delete.Enabled = False
            B_Save.Enabled = False
            Return
        End If
        TB_CreatedAt.Text = Me.Problems(Me.ProblemIndex).CreatedAt
        TB_CreatedBy.Text = Me.Problems(Me.ProblemIndex).CreatedBy
        TB_Duration.Text = Me.Problems(Me.ProblemIndex).Duration
        TB_Message.Text = Me.Problems(Me.ProblemIndex).Message
        TB_Summary.Text = Me.Problems(Me.ProblemIndex).Summary

        B_Next.Enabled = Me.Problems.Count > 1
        B_Ack.Enabled = Me.Problems(Me.ProblemIndex).Acknowledge = Date.MinValue
        B_Delete.Enabled = True
        B_Save.Enabled = True
    End Sub

    ''' <summary>
    ''' Clears the Form
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ClearForm()
        Me.TB_Message.Text = ""
        Me.TB_CreatedBy.Text = ""
        Me.TB_CreatedAt.Text = ""
        Me.TB_Duration.Text = ""
        Me.TB_Summary.Text = ""
    End Sub

    ''' <summary>
    ''' Displays the Baloon
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DisplayProblemBaloon()
        Dim mess As String = ""
        For Each prb In Me.Problems
            If prb.Acknowledge <> Date.MinValue Then Continue For
            mess += prb.Summary
            mess += vbCrLf
        Next
        If mess = "" Then
            NI_Notify.Text = "Com Prob"
            Return
        End If
        NI_Notify.Text = "Com Prob, Issues:" + mess.Split(vbCrLf).Length.ToString
        NI_Notify.ShowBalloonTip(30000, "Current Issues", mess, ToolTipIcon.Info)
    End Sub

#End Region

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        TimerCheckFile_Tick(Nothing, Nothing)
    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        TimerNotify_Tick(Nothing, Nothing)
    End Sub

End Class
